/*
 *  GPSdo - Software for GPSDO
 *  (c) Ad Sekeris 2019
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * File: config_func.h - contains all encoder related functions
 */

byte RS232chip, Si5351chip, Adf4351chip;
byte HardwareSelector, OptionSelector, BaudrateSelector, SiClk0Selector, SiClk1Selector, SiClk2Selector, AdfClkSelector, LcdMeterSelector;
bool EditMode = false;
char MSG_LEVEL[] = "      ";

void Get_Start_Values()
{
  EEPROM.get(0, RS232chip);
  if (RS232chip == 0xFF) {
    RS232chip = 0;
    EEPROM.put(0, RS232chip);
  }
  EEPROM.get(1, NmeaRS232On);
  if (NmeaRS232On == 0xFF) {
    NmeaRS232On = 0;
    EEPROM.put(1, NmeaRS232On);
  }
  EEPROM.get(2, NmeaRS232Baud);
  if (NmeaRS232Baud == 0xFFFFFFFF) {
    NmeaRS232Baud = 9600;
    EEPROM.put(2, NmeaRS232Baud);
  }
  EEPROM.get(6, NmeaUSBOn);
  if (NmeaUSBOn == 0xFF) {
    NmeaUSBOn = 0;
    EEPROM.put(6, NmeaUSBOn);
  }
  EEPROM.get(7, NmeaUSBBaud);
  if (NmeaUSBBaud == 0xFFFFFFFF) {
    NmeaUSBBaud = 9600;
    EEPROM.put(7, NmeaUSBBaud);
  }
  EEPROM.get(11, DebugUSBOn);
  if (DebugUSBOn == 0xFF) {
    DebugUSBOn = 0;
    EEPROM.put(11, DebugUSBOn);
  }
  EEPROM.get(12, DebugUSBBaud);
  if (DebugUSBBaud == 0xFFFFFFFF) {
    DebugUSBBaud = 9600;
    EEPROM.put(12, DebugUSBBaud);
  }

  EEPROM.get(16, Si5351chip);
  if (Si5351chip == 0xFF) {
    Si5351chip = 0;
    EEPROM.put(16, Si5351chip);
  }

  EEPROM.get(17, Adf4351chip);
  if (Adf4351chip == 0xFF) {
    Adf4351chip = 0;
    EEPROM.put(17, Adf4351chip);
  }

  EEPROM.get(18, Si0MHertz);
  if (Si0MHertz == 0xFFFF) {
    Si0MHertz = 25;
    EEPROM.put(18, Si0MHertz);
  }
  EEPROM.get(20, Si0kHertz);
  if (Si0kHertz == 0xFFFF) {
    Si0kHertz = 0;
    EEPROM.put(20, Si0kHertz);
  }
  EEPROM.get(22, Si0Hertz);
  if (Si0Hertz == 0xFFFF) {
    Si0Hertz = 0;
    EEPROM.put(22, Si0Hertz);
  }
  EEPROM.get(24, Si0mHertz);
  if (Si0mHertz == 0xFFFF) {
    Si0mHertz = 0;
    EEPROM.put(24, Si0mHertz);
  }

  EEPROM.get(26, Si1MHertz);
  if (Si1MHertz == 0xFFFF) {
    Si1MHertz = 10;
    EEPROM.put(26, Si1MHertz);
  }
  EEPROM.get(28, Si1kHertz);
  if (Si1kHertz == 0xFFFF) {
    Si1kHertz = 0;
    EEPROM.put(28, Si1kHertz);
  }
  EEPROM.get(30, Si1Hertz);
  if (Si1Hertz == 0xFFFF) {
    Si1Hertz = 0;
    EEPROM.put(30, Si1Hertz);
  }
  EEPROM.get(32, Si1mHertz);
  if (Si1mHertz == 0xFFFF) {
    Si1mHertz = 0;
    EEPROM.put(32, Si1mHertz);
  }

  EEPROM.get(34, Si2MHertz);
  if (Si2MHertz == 0xFFFF) {
    Si2MHertz = 40;
    EEPROM.put(34, Si2MHertz);
  }
  EEPROM.get(36, Si2kHertz);
  if (Si2kHertz == 0xFFFF) {
    Si2kHertz = 0;
    EEPROM.put(36, Si2kHertz);
  }
  EEPROM.get(38, Si2Hertz);
  if (Si2Hertz == 0xFFFF) {
    Si2Hertz = 0;
    EEPROM.put(38, Si2Hertz);
  }
  EEPROM.get(40, Si2mHertz);
  if (Si2mHertz == 0xFFFF) {
    Si2mHertz = 0;
    EEPROM.put(40, Si2mHertz);
  }

  EEPROM.get(42, AdfGHz);
  if (AdfGHz == 0xFFFF) {
    AdfGHz = 0;
    EEPROM.put(42, AdfGHz);
  }
  EEPROM.get(44, AdfMHz);
  if (AdfMHz == 0xFFFF) {
    AdfMHz = 307;
    EEPROM.put(44, AdfMHz);
  }
  EEPROM.get(46, AdfkHz);
  if (AdfkHz == 0xFFFF) {
    AdfkHz = 500;
    EEPROM.put(46, AdfkHz);
  }
  EEPROM.get(48, AdfHz);
  if (AdfHz == 0xFFFF) {
    AdfHz = 0;
    EEPROM.put(48, AdfHz);
  }

  EEPROM.get(50, Si0Level);
  if (Si0Level == 0xFF) {
    Si0Level = 0;
    EEPROM.put(50, Si0Level);
  }

  EEPROM.get(51, Si1Level);
  if (Si1Level == 0xFF) {
    Si1Level = 0;
    EEPROM.put(51, Si1Level);
  }

  EEPROM.get(52, Si2Level);
  if (Si2Level == 0xFF) {
    Si2Level = 0;
    EEPROM.put(52, Si2Level);
  }

  EEPROM.get(53, AdfLevel);
  if (AdfLevel == 0xFF) {
    AdfLevel = 0;
    EEPROM.put(53, AdfLevel);
  }

  EEPROM.get(54, LcdTot);
  if (LcdTot == 0xFF) {
    LcdTot = 0;
    EEPROM.put(54, LcdTot);
  }

  EEPROM.get(55, MeterMaxVal);
  if (MeterMaxVal == 0xFFFF) {
    MeterMaxVal = 32;
    EEPROM.put(55, MeterMaxVal);
  }

  EEPROM.get(57, PllMod);
  if ((PllMod == 0xFF) or (PllMod > 0x04)) {
    PllMod = 0x00;
    EEPROM.put(57, PllMod);
 }

  if (Si5351chip == 1){
    MaxScreen = BaseScreen + 3;
    if (Adf4351chip == 1) MaxScreen ++;
  }
}

void RotateLCDCalibration (byte Rotary) {
  char TempField[] = "    ";
  if (Rotary == 1) {
    if (MeterMaxVal <1024) MeterMaxVal ++;
  }
  else {
    if (MeterMaxVal > 0) MeterMaxVal --;
  }
  lcd.setCursor(0, 3);
  sprintf(TempField, "%4d", round(WeighedHigh));
  lcd.print(TempField);
  lcd.setCursor(6, 3);
  lcd.print("Eeprom:");
  sprintf(TempField, "%4d", MeterMaxVal);
  lcd.print(TempField);
}

void LcdMeterCalibration()
{
  char TempField[] = "    ";
  switch (LcdMeterSelector) {
  case 0:
    LcdMeterSelector ++;
      EditMode = true;
      #if (LCD_2004)
        lcd.setCursor(0, 3);
        sprintf(TempField, "%4d", round(WeighedHigh));
        lcd.print(TempField);
        lcd.setCursor(6, 3);
        lcd.print("Eeprom:");
        sprintf(TempField, "%4d", MeterMaxVal);
        lcd.print(TempField);
        lcd.blink();
      #endif
    break;
  case 1:
    LcdMeterSelector = 0;
      EditMode = false;
      #if (LCD_2004)
        lcd.setCursor(0,3);
        lcd.noBlink();
        lcd.print(MSG_EMPTY_LINE);
      #endif
      unsigned int TempVal;
      if (EEPROM.get(55, TempVal)!= MeterMaxVal) EEPROM.put(55, MeterMaxVal);
    break;
  }    
}


unsigned int RotatemHzFrequency (byte Rotary, unsigned int InValue, byte ScreenCol, byte ScreenRow, byte CursorCol, byte CursorRow, unsigned int MinVal, unsigned int MaxVal)
{
  unsigned int ReturnValue;
  char TempTxt[] = "  ";
  ReturnValue = InValue;
  if (Rotary == 1){
    ReturnValue ++;
    if (ReturnValue > MaxVal) ReturnValue = MinVal;
  }
  else {
    ReturnValue --;
    if (ReturnValue < MinVal || ReturnValue == 0xFFFF) ReturnValue = MaxVal;
  }
  sprintf(TempTxt, "%02d", ReturnValue);
  #if (LCD_2004)
    lcd.setCursor(ScreenCol, ScreenRow);
    lcd.print(TempTxt);
    if(CursorCol != 0 || CursorRow != 0) lcd.setCursor(CursorCol, CursorRow);
  #endif
  return ReturnValue;
}

unsigned int RotatekHzFrequency (byte Rotary, unsigned int InValue, byte ScreenCol, byte ScreenRow, byte CursorCol, byte CursorRow, unsigned int MinVal, unsigned int MaxVal)
{
  // also used for Hz adjustment. So RotateHzFrequency does not exist
  unsigned int ReturnValue;
  char TempTxt[] = "   ";
  ReturnValue = InValue;
  if (Rotary == 1){
    ReturnValue ++;
    if (ReturnValue > MaxVal) ReturnValue = MinVal;
  }
  else {
    ReturnValue --;
    if (ReturnValue < MinVal || ReturnValue == 0xFFFF) ReturnValue = MaxVal;
  }
  sprintf(TempTxt, "%03d", ReturnValue);
  #if (LCD_2004)
    lcd.setCursor(ScreenCol, ScreenRow);
    lcd.print(TempTxt);
    if(CursorCol != 0 || CursorRow != 0) lcd.setCursor(CursorCol, CursorRow);
  #endif
  return ReturnValue;
}

unsigned int RotateMHzFrequency (byte Rotary, unsigned int InValue, byte ScreenCol, byte ScreenRow, byte CursorCol, byte CursorRow, unsigned int MinVal, unsigned int MaxVal)
{
  unsigned int ReturnValue;
  char TempTxt[] = "   ";
  ReturnValue = InValue;
  if (Rotary == 1){
    ReturnValue ++;
    if (ReturnValue > MaxVal) ReturnValue = MinVal;
  }
  else {
    ReturnValue --;
    if (ReturnValue < MinVal || ReturnValue == 0xFFFF) ReturnValue = MaxVal;
  }
  sprintf(TempTxt, "%3d", ReturnValue);
  #if (LCD_2004)
    lcd.setCursor(ScreenCol, ScreenRow);
    lcd.print(TempTxt);
    if(CursorCol != 0 || CursorRow != 0) lcd.setCursor(CursorCol, CursorRow);
  #endif
  return ReturnValue;
}

byte RotateClkLevel (byte Rotary, byte InValue, byte ScreenCol, byte ScreenRow, byte CursorCol, byte CursorRow, unsigned int MinVal, unsigned int MaxVal)
{
  byte ReturnValue;
  ReturnValue = InValue;
  if (Rotary == 1){
    ReturnValue ++;
    if (ReturnValue > MaxVal) ReturnValue = MinVal;
  }
  else {
    ReturnValue --;
    if (ReturnValue < MinVal || ReturnValue == 0xFF) ReturnValue = MaxVal;
  }
  #if (LCD_2004)
    lcd.setCursor(ScreenCol, ScreenRow);
    if (ReturnValue == 0) lcd.print(MSG_OFF); 
    else if (ReturnValue == 1) lcd.print(MSG_LOW);
    else if (ReturnValue == 2) lcd.print(MSG_MID);
    else if (ReturnValue == 3) lcd.print(MSG_HIGH);
    if(CursorCol != 0 || CursorRow != 0) lcd.setCursor(CursorCol, CursorRow);
  #endif
  return ReturnValue;
}

void RotateAdfkHz (byte Rotary)
{
  char TempTxt[] = "    ";
  if (Rotary == 1){
    AdfkHz ++;
    if (AdfkHz > 999) AdfkHz = 0;
  }
  else {
    AdfkHz --;
    if (AdfkHz < 0 || AdfkHz == 0xFFFF) AdfkHz = 999;
  }
  sprintf(TempTxt, "%03d", AdfkHz);
  #if (LCD_2004)
    lcd.setCursor(5,2);
    lcd.print(TempTxt);
    lcd.setCursor(8, 3);
  #endif
}

void RotateAdfMHz (byte Rotary)
{
  char TempTxt[] = "    ";
  if (Rotary == 1){
    AdfMHz ++;
    if (AdfMHz > 4399) AdfMHz = 35;
  }
  else {
    AdfMHz --;
    if (AdfMHz < 35 || AdfMHz == 0xFFFF) AdfMHz = 4399;
  }
  sprintf(TempTxt, "%4d", AdfMHz);
  #if (LCD_2004)
    lcd.setCursor(0,2);
    lcd.print(TempTxt);
    lcd.setCursor(4, 3);
  #endif
}

void RotateAdfLevel (byte Rotary)
{
  if (Rotary == 1){
    AdfLevel ++;
    if (AdfLevel > 4) AdfLevel = 0;
  }
  else {
    AdfLevel --;
    if (AdfLevel<0 || AdfLevel == 0xFF) AdfLevel = 4;
  }
//  if (AdfLevel == 0) dds.setRFOut(0);
//  else {
//    dds.setRFPower(AdfLevel-1);
//    dds.setRFOut(1);
//  }
//  dds.update();
  #if (LCD_2004)
    lcd.setCursor(0, 1);
    lcd.print(MSG_OUTPUT);
    lcd.blink();
    if (AdfLevel == 0) lcd.print(MSG_OFF); 
    else {
      sprintf(MSG_LEVEL, "%1d   ", AdfLevel -1);
      lcd.print(MSG_LEVEL);
    }
  #endif
}

void SelectSiClk0()
{
  switch (SiClk0Selector) {
  case 0:
    SiClk0Selector ++;
      EditMode = true;
      #if (LCD_2004)
        lcd.setCursor(0, 1);
        lcd.print(MSG_OUTPUT);
        lcd.blink();
        if (Si0Level == 0) lcd.print(MSG_OFF); 
        else if (Si0Level == 1) lcd.print(MSG_LOW);
        else if (Si0Level == 2) lcd.print(MSG_MID);
        else if (Si0Level == 3) lcd.print(MSG_HIGH);
      #endif
    break;
  case 1:
    SiClk0Selector ++;
      EditMode = true;
      #if (LCD_2004)
        lcd.setCursor(0,3);
        lcd.blink();
        lcd.print("MHz");
      #endif
    break;
  case 2:
    SiClk0Selector ++;
      EditMode = true;
      #if (LCD_2004)
        lcd.setCursor(0,3);
        lcd.blink();
        lcd.print("    kHz");
      #endif
    break;
  case 3:
    SiClk0Selector ++;
      EditMode = true;
      #if (LCD_2004)
        lcd.setCursor(0,3);
        lcd.blink();
        lcd.print("         Hz");
      #endif
    break;
  case 4:
    SiClk0Selector ++;
      EditMode = true;
      #if (LCD_2004)
        lcd.setCursor(0,3);
        lcd.blink();
        lcd.print("              ");
      #endif
    break;
  case 5:
    SiClk0Selector = 0;
      EditMode = false;
      #if (LCD_2004)
        lcd.setCursor(0,3);
        lcd.noBlink();
        lcd.print("            ");
      #endif
      unsigned int TempVal;
      EEPROM.update(50, Si0Level);
      Si0Freq = (Si0MHertz * 100000000ull)+(Si0kHertz*100000)+(Si0Hertz*100)+Si0mHertz;
      si5351.set_freq(Si0Freq, SI5351_CLK0);
      //si5351.update_status();
      if (EEPROM.get(18, TempVal)!= Si0MHertz) EEPROM.put(18, Si0MHertz);
      if (EEPROM.get(20, TempVal)!= Si0kHertz) EEPROM.put(20, Si0kHertz);
      if (EEPROM.get(22, TempVal)!= Si0Hertz) EEPROM.put(22, Si0Hertz);
      if (EEPROM.get(24, TempVal)!= Si0mHertz) EEPROM.put(24, Si0mHertz);
    break;
  }    
}

void SelectSiClk1()
{
  switch (SiClk1Selector) {
  case 0:
    SiClk1Selector ++;
      EditMode = true;
      #if (LCD_2004)
        lcd.setCursor(0, 1);
        lcd.print(MSG_OUTPUT);
        lcd.blink();
        if (Si1Level == 0) lcd.print(MSG_OFF); 
        else if (Si1Level == 1) lcd.print(MSG_LOW);
        else if (Si1Level == 2) lcd.print(MSG_MID);
        else if (Si1Level == 3) lcd.print(MSG_HIGH);
      #endif
    break;
  case 1:
    SiClk1Selector ++;
      EditMode = true;
      #if (LCD_2004)
        lcd.setCursor(0,3);
        lcd.blink();
        lcd.print("MHz");
      #endif
    break;
  case 2:
    SiClk1Selector ++;
      EditMode = true;
      #if (LCD_2004)
        lcd.setCursor(0,3);
        lcd.blink();
        lcd.print("    kHz");
      #endif
    break;
  case 3:
    SiClk1Selector ++;
      EditMode = true;
      #if (LCD_2004)
        lcd.setCursor(0,3);
        lcd.blink();
        lcd.print("         Hz");
      #endif
    break;
  case 4:
    SiClk1Selector ++;
      EditMode = true;
      #if (LCD_2004)
        lcd.setCursor(0,3);
        lcd.blink();
        lcd.print("              ");
      #endif
    break;
  case 5:
    SiClk1Selector = 0;
      EditMode = false;
      #if (LCD_2004)
        lcd.setCursor(0,3);
        lcd.noBlink();
        lcd.print("            ");
      #endif
      unsigned int TempVal;
      EEPROM.update(51, Si1Level);
      Si1Freq = (Si1MHertz * 100000000ull)+(Si1kHertz*100000)+(Si1Hertz*100)+Si1mHertz;
      si5351.set_freq(Si1Freq, SI5351_CLK1);
      //si5351.update_status();
      if (EEPROM.get(26, TempVal)!= Si1MHertz) EEPROM.put(26, Si1MHertz);
      if (EEPROM.get(28, TempVal)!= Si1kHertz) EEPROM.put(28, Si1kHertz);
      if (EEPROM.get(30, TempVal)!= Si1Hertz) EEPROM.put(30, Si1Hertz);
      if (EEPROM.get(32, TempVal)!= Si1mHertz) EEPROM.put(32, Si1mHertz);
    break;
  }    
}
void SelectSiClk2()
{
  switch (SiClk2Selector) {
  case 0:
    SiClk2Selector ++;
      EditMode = true;
      #if (LCD_2004)
        lcd.setCursor(0, 1);
        lcd.print(MSG_OUTPUT);
        lcd.blink();
        if (Si2Level == 0) lcd.print(MSG_OFF); 
        else if (Si2Level == 1) lcd.print(MSG_LOW);
        else if (Si2Level == 2) lcd.print(MSG_MID);
        else if (Si2Level == 3) lcd.print(MSG_HIGH);
      #endif
    break;
  case 1:
    SiClk2Selector ++;
      EditMode = true;
      #if (LCD_2004)
        lcd.setCursor(0,3);
        lcd.blink();
        lcd.print("MHz");
      #endif
    break;
  case 2:
    SiClk2Selector ++;
      EditMode = true;
      #if (LCD_2004)
        lcd.setCursor(0,3);
        lcd.blink();
        lcd.print("    kHz");
      #endif
    break;
  case 3:
    SiClk2Selector ++;
      EditMode = true;
      #if (LCD_2004)
        lcd.setCursor(0,3);
        lcd.blink();
        lcd.print("         Hz");
      #endif
    break;
  case 4:
    SiClk2Selector ++;
      EditMode = true;
      #if (LCD_2004)
        lcd.setCursor(0,3);
        lcd.blink();
        lcd.print("              ");
      #endif
    break;
  case 5:
    SiClk2Selector = 0;
      EditMode = false;
      #if (LCD_2004)
        lcd.setCursor(0,3);
        lcd.noBlink();
        lcd.print("            ");
      #endif
      unsigned int TempVal;
      EEPROM.update(52, Si2Level);
      Si2Freq = (Si2MHertz * 100000000ull)+(Si2kHertz*100000)+(Si2Hertz*100)+Si2mHertz;
      si5351.set_freq(Si2Freq, SI5351_CLK2);
      //si5351.update_status();
      if (EEPROM.get(34, TempVal)!= Si2MHertz) EEPROM.put(34, Si2MHertz);
      if (EEPROM.get(36, TempVal)!= Si2kHertz) EEPROM.put(36, Si2kHertz);
      if (EEPROM.get(38, TempVal)!= Si2Hertz) EEPROM.put(38, Si2Hertz);
      if (EEPROM.get(40, TempVal)!= Si2mHertz) EEPROM.put(40, Si2mHertz);
    break;
  }      
}
void SelectAdfClk()
{
  switch (AdfClkSelector) {
  case 0:
    AdfClkSelector ++;
    EditMode = true;
    #if (LCD_2004)
      lcd.setCursor(0, 1);
      lcd.print(MSG_OUTPUT);
      lcd.blink();
      if (AdfLevel == 0) lcd.print(MSG_OFF); 
      else {
        sprintf(MSG_LEVEL, "%1d   ", AdfLevel-1);
        lcd.print(MSG_LEVEL);
      }
    #endif
    break;
  case 1:
    AdfClkSelector ++;
    EditMode = true;
    #if (LCD_2004)
      lcd.setCursor(0,3);
      lcd.blink();
      lcd.print(" MHz");
    #endif
    break;
  case 2:
    AdfClkSelector ++;
    EditMode = true;
    #if (LCD_2004)
      lcd.setCursor(0,3);
      lcd.blink();
      lcd.print("     kHz");
    #endif
    break;
  case 3:
    AdfClkSelector = 0;
    EditMode = false;
    #if (LCD_2004)
      lcd.setCursor(0,3);
      lcd.noBlink();
      lcd.print("            ");
    #endif
    AdfFreq = (AdfMHz*1000000ull)+(AdfkHz*1000ull);
    dds.setFreq(AdfFreq, AdfStep);
    dds.update();
    unsigned int TempVal;
    EEPROM.update(53, AdfLevel);
    if (EEPROM.get(42, TempVal)!= AdfGHz) EEPROM.put(42, AdfGHz);
    if (EEPROM.get(44, TempVal)!= AdfMHz) EEPROM.put(44, AdfMHz);
    if (EEPROM.get(46, TempVal)!= AdfkHz) EEPROM.put(46, AdfkHz);
    if (EEPROM.get(48, TempVal)!= AdfHz) EEPROM.put(48, AdfHz);
    break;
  }
}

void SelectBaudrate()
{
  unsigned long TempBaud;
  if(RS232chip ==1) {
    switch (BaudrateSelector) {
    case 0:
      BaudrateSelector ++;
      EditMode = true;
      #if (LCD_2004)
        lcd.setCursor(0,0);
        lcd.print(MSG_USB_NMEA);
        lcd.setCursor(12,0);
        lcd.blink();
        if (NmeaUSBOn == 1) lcd.print(MSG_ON); else lcd.print(MSG_OFF);
      #endif
      break;
    case 1:
      BaudrateSelector ++;
      #if (LCD_2004)
        lcd.setCursor(1,1);
        lcd.print(MSG_BAUD_RATE);
        lcd.setCursor(12,1);
        lcd.blink();
        switch (NmeaUSBBaud){
          case 9600:
            lcd.print(MSG_BAUD_9600);
            break;
          case 19200:
            lcd.print(MSG_BAUD_19200);
            break;
          case 38400:
            lcd.print(MSG_BAUD_38400);
            break;
          case 57600:
            lcd.print(MSG_BAUD_57600);
            break;
          case 115200:
            lcd.print(MSG_BAUD_115200);
            break;
        }
      #endif
      break;
    case 2:
      BaudrateSelector ++;
      #if (LCD_2004)
        lcd.setCursor(0,2);
        lcd.print(MSG_RS232C_NMEA);
        lcd.setCursor(12,2);
        lcd.blink();
        if (NmeaRS232On == 1) lcd.print(MSG_ON); else lcd.print(MSG_OFF);
      #endif
      break;
    case 3:
      BaudrateSelector ++;
      #if (LCD_2004)
        lcd.setCursor(1,3);
        lcd.print(MSG_BAUD_RATE);
        lcd.setCursor(12,3);
        lcd.blink();
        switch (NmeaRS232Baud){
          case 4800:
            lcd.print(MSG_BAUD_4800);
            break;
          case 9600:
            lcd.print(MSG_BAUD_9600);
            break;
          case 19200:
            lcd.print(MSG_BAUD_19200);
            break;
          case 38400:
            lcd.print(MSG_BAUD_38400);
            break;
          case 57600:
            lcd.print(MSG_BAUD_57600);
            break;
          case 115200:
            lcd.print(MSG_BAUD_115200);
            break;
        }
      #endif
      break;
    case 4:
      BaudrateSelector = 0;
      EditMode = false;;
      #if (LCD_2004)
        lcd.noBlink();
      #endif
      EEPROM.update(1, NmeaRS232On);
      if (EEPROM.get(2, TempBaud)!= NmeaRS232Baud) EEPROM.put(2, NmeaRS232Baud);
      EEPROM.update(6, NmeaUSBOn);
      if (EEPROM.get(7, TempBaud)!=NmeaUSBBaud) EEPROM.put(7, NmeaUSBBaud);
      break;
    } 
  }  
  else {
    switch (BaudrateSelector) {
    case 0:
      BaudrateSelector ++;
      EditMode = true;
      #if (LCD_2004)
        lcd.setCursor(0,0);
        lcd.print(MSG_USB_NMEA);
        lcd.setCursor(12,0);
        lcd.blink();
        if (NmeaUSBOn == 1) lcd.print(MSG_ON); else lcd.print(MSG_OFF);
      #endif
      break;
    case 1:
      BaudrateSelector ++;
      #if (LCD_2004)
        lcd.setCursor(1,1);
        lcd.print(MSG_BAUD_RATE);
        lcd.setCursor(12,1);
        lcd.blink();
        switch (NmeaUSBBaud){
          case 9600:
            lcd.print(MSG_BAUD_9600);
            break;
          case 19200:
            lcd.print(MSG_BAUD_19200);
            break;
          case 38400:
            lcd.print(MSG_BAUD_38400);
            break;
          case 57600:
            lcd.print(MSG_BAUD_57600);
            break;
          case 115200:
            lcd.print(MSG_BAUD_115200);
            break;
        }
      #endif
      break;
    case 2:
      BaudrateSelector = 0;
      EditMode = false;;
      #if (LCD_2004)
        lcd.noBlink();
      #endif
      EEPROM.update(6, NmeaUSBOn);
      if (EEPROM.get(7, TempBaud)!=NmeaUSBBaud) EEPROM.put(7, NmeaUSBBaud);
      break;
    } 
  }  
}

void SelectUSBBaudRate(byte Rotation)
{
  if (Rotation == 1){
    switch (NmeaUSBBaud) {
      case 9600:
        NmeaUSBBaud = 19200;
        break;
      case 19200:
        NmeaUSBBaud = 38400;
        break;
      case 38400:
        NmeaUSBBaud = 57600;
        break;
      case 57600:
        NmeaUSBBaud = 115200;
      break;
      case 115200:
        NmeaUSBBaud = 9600;
        break;
    }
  }
  else {
    switch (NmeaUSBBaud) {
      case 9600:
        NmeaUSBBaud = 115200;
        break;
      case 19200:
        NmeaUSBBaud = 9600;
        break;
      case 38400:
        NmeaUSBBaud = 19200;
        break;
      case 57600:
        NmeaUSBBaud = 38400;
      break;
      case 115200:
        NmeaUSBBaud = 57600;
        break;
    }
  }
  #if (LCD_2004)
    lcd.setCursor(12,1);
    switch (NmeaUSBBaud){
      case 9600:
        lcd.print(MSG_BAUD_9600);
        break;
      case 19200:
        lcd.print(MSG_BAUD_19200);
        break;
      case 38400:
        lcd.print(MSG_BAUD_38400);
        break;
      case 57600:
        lcd.print(MSG_BAUD_57600);
        break;
      case 115200:
        lcd.print(MSG_BAUD_115200);
        break;
    }
  #endif
  if (USB_DEBUG or NmeaUSBOn == 1) Serial.begin (NmeaUSBBaud); else Serial.end();
}

void SelectRS232BaudRate(byte Rotation)
{
  if (Rotation == 1){
    switch (NmeaRS232Baud) {
      case 4800:
        NmeaRS232Baud = 9600;
        break;
      case 9600:
        NmeaRS232Baud = 19200;
        break;
      case 19200:
        NmeaRS232Baud = 38400;
        break;
      case 38400:
        NmeaRS232Baud = 57600;
        break;
      case 57600:
        NmeaRS232Baud = 115200;
      break;
      case 115200:
        NmeaRS232Baud = 4800;
        break;
    }
  }
  else {
    switch (NmeaRS232Baud) {
      case 4800:
        NmeaRS232Baud = 115200;
        break;
      case 9600:
        NmeaRS232Baud = 4800;
        break;
      case 19200:
        NmeaRS232Baud = 9600;
        break;
      case 38400:
        NmeaRS232Baud = 19200;
        break;
      case 57600:
        NmeaRS232Baud = 38400;
      break;
      case 115200:
        NmeaRS232Baud = 57600;
        break;
    }
  }
  #if (LCD_2004)
    lcd.setCursor(12,3);
    switch (NmeaRS232Baud){
      case 4800:
        lcd.print(MSG_BAUD_4800);
        break;
      case 9600:
        lcd.print(MSG_BAUD_9600);
        break;
      case 19200:
        lcd.print(MSG_BAUD_19200);
        break;
      case 38400:
        lcd.print(MSG_BAUD_38400);
        break;
      case 57600:
        lcd.print(MSG_BAUD_57600);
        break;
      case 115200:
        lcd.print(MSG_BAUD_115200);
        break;
    }
  #endif  
  if (NmeaRS232On == 1 && RS232chip == 1) Serial3.begin (NmeaRS232Baud); else Serial3.end();
}

void SelectHardware()
{
  switch (HardwareSelector) {
  case 0:
    HardwareSelector ++;
    EditMode = true;
    #if (LCD_2004)
      lcd.setCursor(0,1);
      lcd.print(MSG_HARDWARE_RS232C);
      lcd.blink();
      if (RS232chip == 1) lcd.print(MSG_ON); else lcd.print(MSG_OFF);
    #endif
    break;
  case 1: //RS232
    HardwareSelector ++;
    EditMode = true;
    #if (LCD_2004)
      lcd.setCursor(0,2);
      lcd.print(MSG_HARDWARE_SI5351);
      lcd.blink();
      if (Si5351chip == 1) lcd.print(MSG_ON); else lcd.print(MSG_OFF);
    #endif
    break;
  case 2:  //Si5351
    HardwareSelector ++;
    EditMode = true;
    #if (LCD_2004)
      lcd.setCursor(0,3);
      lcd.print(MSG_HARDWARE_ADF4351);
      lcd.blink();
      if (Adf4351chip == 1) lcd.print(MSG_ON); else lcd.print(MSG_OFF);
    #endif
    break;
  case 3:  //Adf4351
    HardwareSelector = 0;
    EditMode = false;
    #if (LCD_2004)
      lcd.noBlink();
    #endif
    if (Si5351chip == 1){
      MaxScreen = BaseScreen + 3;
      if (Adf4351chip == 1) MaxScreen = BaseScreen + 4;
    }
    else {
      MaxScreen = BaseScreen;
      Adf4351chip = 0;
    }
    #if (LCD_2004)
      lcd.setCursor(0, 0);
      lcd.print(MSG_HARDWARE_OPTION);
      lcd.setCursor(0,1);
      lcd.print(MSG_HARDWARE_RS232C);
      if (RS232chip == 1) lcd.print(MSG_ON); else lcd.print(MSG_OFF);
      lcd.setCursor(0,2);
      lcd.print(MSG_HARDWARE_SI5351);
      if (Si5351chip == 1) lcd.print(MSG_ON); else lcd.print(MSG_OFF);
      lcd.setCursor(0,3);
      lcd.print(MSG_HARDWARE_ADF4351);
      if (Adf4351chip == 1) lcd.print(MSG_ON); else lcd.print(MSG_OFF);
    #endif
    //update EEprom
    EEPROM.update(0, RS232chip);
    EEPROM.update(16, Si5351chip);
    EEPROM.update(17, Adf4351chip);  
    break;
  }
}

void ToggleSelectedHardware() 
{
  switch (HardwareSelector) {
  case 1:
    if (RS232chip == 1) RS232chip = 0; else RS232chip = 1;
    #if (LCD_2004)
      lcd.setCursor(0,1);
      lcd.print(MSG_HARDWARE_RS232C);
      if (RS232chip == 1) lcd.print(MSG_ON); else lcd.print(MSG_OFF);
    #endif
    break;
  case 2:
    if (Si5351chip == 1) Si5351chip =0; else Si5351chip = 1;
    #if (LCD_2004)
      lcd.setCursor(0,2);
      lcd.print(MSG_HARDWARE_SI5351);
      if (Si5351chip == 1) lcd.print(MSG_ON); else lcd.print(MSG_OFF);
    #endif
    Si0Freq = (Si0MHertz * 100000000ull)+(Si0kHertz*100000)+(Si0Hertz*100)+Si0mHertz;
    Si1Freq = (Si1MHertz * 100000000ull)+(Si1kHertz*100000)+(Si1Hertz*100)+Si1mHertz;
    Si2Freq = (Si2MHertz * 100000000ull)+(Si2kHertz*100000)+(Si2Hertz*100)+Si2mHertz;
    si5351.set_freq(Si0Freq, SI5351_CLK0);
    if (Si5351chip == 1) {
      if (Si0Level == 0) {
        si5351.output_enable(SI5351_CLK0, 0); 
      }
      else {
        si5351.drive_strength(SI5351_CLK0, Si0Level - 1);
        si5351.output_enable(SI5351_CLK0, 1);       
      }
      si5351.set_freq(Si1Freq, SI5351_CLK1);
      if (Si1Level == 0) {
        si5351.output_enable(SI5351_CLK1, 0); 
      }
      else {
        si5351.drive_strength(SI5351_CLK1, Si1Level - 1);
        si5351.output_enable(SI5351_CLK1, 1);       
      }
      si5351.set_freq(Si2Freq, SI5351_CLK2);
      if (Si2Level == 0) {
        si5351.output_enable(SI5351_CLK2, 0); 
      }
      else {
        si5351.drive_strength(SI5351_CLK2, Si2Level - 1);
        si5351.output_enable(SI5351_CLK2, 1);       
      }
      if (Adf4351chip == 1) {
        AdfFreq = (AdfMHz*1000000ull)+(AdfkHz*1000ull);
        dds.setFreq(AdfFreq, AdfStep);
        if (AdfLevel > 0) {
          dds.setRFOut(1);
          dds.setRFPower(AdfLevel-1);
        }
        else dds.setRFOut(0);   
        dds.update();
      }
    }
    else
    {
      si5351.output_enable(SI5351_CLK0, 0);       
      si5351.output_enable(SI5351_CLK1, 0); 
      si5351.output_enable(SI5351_CLK2, 0); 
      dds.setRFOut(0);   
      dds.update();
      Adf4351chip = 0;
      MaxScreen = BaseScreen;
    }
    break;
  case 3:
    if (Si5351chip == 1) {
      if (Adf4351chip == 1) Adf4351chip = 0; else Adf4351chip = 1;
      #if (LCD_2004)
        lcd.setCursor(0,3);
        lcd.print(MSG_HARDWARE_ADF4351);
        if (Adf4351chip == 1) lcd.print(MSG_ON); else lcd.print(MSG_OFF);
      #endif
      AdfFreq = (AdfMHz*1000000ull)+(AdfkHz*1000ull);
      dds.setFreq(AdfFreq, AdfStep);
      if (AdfLevel > 0) {
        dds.setRFOut(1);
        dds.setRFPower(AdfLevel-1);
      }
      else dds.setRFOut(0);   
      dds.update();
    }
    break;  
  } 
}

void TogglePllMod(byte Rotation)
{
  if (Rotation == 1){
    PllMod ++;
    if (PllMod > 4) PllMod = 0;
  }
  else {
    PllMod --;
    if (PllMod <0 || PllMod == 0xFF) PllMod = 4;
  }

  #if (LCD_2004)
    lcd.setCursor(11,2);
    switch (PllMod){
      case 0:
        lcd.print(PLL_MOD_0);
        break;
      case 1:
        lcd.print(PLL_MOD_1);
        break;
      case 2:
        lcd.print(PLL_MOD_2);
        break;
      case 3:
        lcd.print(PLL_MOD_3);
        break;
      case 4:
        lcd.print(PLL_MOD_4);
        break;
    }
  #endif
}

void ToggleUSBNmea()
{
  if (NmeaUSBOn == 1) NmeaUSBOn = 0; else NmeaUSBOn = 1;
  #if (LCD_2004)
    lcd.setCursor(12,0);
    if (NmeaUSBOn == 1) lcd.print(MSG_ON); else lcd.print(MSG_OFF);
  #endif
  if (USB_DEBUG or NmeaUSBOn == 1) Serial.begin (NmeaUSBBaud); else Serial.end();
}

void ToggleRS232Nmea()
{
  if (NmeaRS232On == 1) NmeaRS232On = 0; else NmeaRS232On = 1;
  #if (LCD_2004)
    lcd.setCursor(12,2);
    if (NmeaRS232On == 1) lcd.print(MSG_ON); else lcd.print(MSG_OFF);
  #endif
  if (NmeaRS232On == 1 && RS232chip == 1) Serial2.begin (NmeaRS232Baud); else Serial2.end();
}

void SelectLcdTot(byte Rotation)
{
  if (Rotation == 1){
    LcdTot ++;
    if (LcdTot > 8) LcdTot = 0;
  }
  else {
    LcdTot --;
    if (LcdTot <0 || LcdTot == 0xFF) LcdTot = 8;
  }
  #if (LCD_2004)
    lcd.setCursor(11,1);
    switch (LcdTot){
      case 0:
        lcd.print(MSG_LCD_TOT_OFF);
        break;
      case 1:
        lcd.print(MSG_LCD_TOT_01MIN);
        break;
      case 2:
        lcd.print(MSG_LCD_TOT_02MIN);
        break;
      case 3:
        lcd.print(MSG_LCD_TOT_03MIN);
        break;
      case 4:
        lcd.print(MSG_LCD_TOT_04MIN);
        break;
      case 5:
        lcd.print(MSG_LCD_TOT_05MIN);
        break;
      case 6:
        lcd.print(MSG_LCD_TOT_10MIN);
        break;
      case 7:
        lcd.print(MSG_LCD_TOT_15MIN);
        break;
      case 8:
        lcd.print(MSG_LCD_TOT_30MIN);
        break;
    }
  #endif  
}

void SelectOption() 
{
  switch (OptionSelector) {
  case 0:
    OptionSelector ++;
    EditMode = true;
    #if (LCD_2004)
      lcd.setCursor(0,1);
      lcd.print(MSG_LCD_TOT);
      lcd.blink();
    switch (LcdTot){
      case 0:
        lcd.print(MSG_LCD_TOT_OFF);
        break;
      case 1:
        lcd.print(MSG_LCD_TOT_01MIN);
        break;
      case 2:
        lcd.print(MSG_LCD_TOT_02MIN);
        break;
      case 3:
        lcd.print(MSG_LCD_TOT_03MIN);
        break;
      case 4:
        lcd.print(MSG_LCD_TOT_04MIN);
        break;
      case 5:
        lcd.print(MSG_LCD_TOT_05MIN);
        break;
      case 6:
        lcd.print(MSG_LCD_TOT_10MIN);
        break;
      case 7:
        lcd.print(MSG_LCD_TOT_15MIN);
        break;
      case 8:
        lcd.print(MSG_LCD_TOT_30MIN);
        break;
    }
    #endif
    break;
  case 1:
    //update EEprom
    EEPROM.update(54, LcdTot);
    OptionSelector ++;
    EditMode = true;
    #if (LCD_2004)
      lcd.setCursor(0,2);
      lcd.print(MSG_LCD_PLL);
      lcd.blink();
    switch (PllMod){
      case 0:
        lcd.print(PLL_MOD_0);
        break;
      case 1:
        lcd.print(PLL_MOD_1);
        break;
      case 2:
        lcd.print(PLL_MOD_2);
        break;
      case 3:
        lcd.print(PLL_MOD_3);
        break;
      case 4:
        lcd.print(PLL_MOD_4);
        break;
    }
    #endif
    break;
  case 2: 
    //update EEprom
    EEPROM.update(57, PllMod);
    OptionSelector = 0;
    EditMode = false;
    #if (LCD_2004)
      lcd.noBlink();
    #endif
    break;
  }  
}

//eof
