/*
 *  GPSdo - Software for GPSDO
 *  (c) Ad Sekeris 2019
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * File: configuration.h - contains all hardware settings
 */

  // GPSDO2 and GPSDO3
  // Screen0 = utc time and date
  // Screen1 = local time, date, timezone and daylight saving
  // Screen2 = UTC and local time
  // Screen3 = GPS location and locator grid. Decimal notation.
  // Screen4 = GPS location and locator grig. Degree, Minutes and Seconds notation.
  // Screen5 = Hardware runtime and gps fixed time
  // Screen6 = Fase indicator
  // Screen7 = Hardware options
  // Screen8 = NMEA output settings
  // Screen9 = Fun settings in eeprom
  // Extra for GPSDO4
  // Screen10 = Si5351 Clk0
  // Screen11 = Si5351 Clk1
  // Screen12 = Si5351 Clk2
  // Extra for ADF4351 print
  // Screen13 = ADF4351
  byte BaseScreen = 9, MaxScreen = 9;

  // Special define for PA0FRI to move analog input from A0 to A1
  // Remove the 2 slashes on the next line to enable this mod
  //#define PA0FRI
  
  // Language selector - only one should be true
  #define LANG_GB true
  #define LANG_NL false

  // Defined timezones
  // CE - Central european time (Frankfurt, Paris, Amsterdam) utc+1 including DST
  // EE - Eastern european time (Athens) utc+2 including DST
  // UK - United Kingdom (London, Belfast) utc including DST
  //
  #define LOCAL_TIME_ZONE_CE true
  #define LOCAL_TIME_ZONE_EE false
  #define LOCAL_TIME_ZONE_UK false
  
  // PLL gemodificeerd naar 100kHz
  byte PllMod;

  // Select debug on off -
  // false - All NMEA messages are printed on the USB serial
  // true  - Program debug messages are printed on the USB serial
  #define USB_DEBUG false
  byte DebugUSBOn;
  unsigned long DebugUSBBaud;

// USB serial NMEA otput settings
  byte NmeaUSBOn;
  unsigned long NmeaUSBBaud;
  
// RS232_C settings
  byte NmeaRS232On;
  unsigned long NmeaRS232Baud;

  // Display settings.
  #define LCD_I2C_addr 0x27  // bus address of the I2C lcd display
  #define OLED_I2C_addr 0x78  // bus address of the I2C oled display
  // Select only one of the following items true.
  #define LCD_2004 true
  #define OLED_128x64 false
  #define NEXTION_35 false
  byte LcdTot = 0 ;
    
  #define NEXTION_size 35  // Size of the nextion display
  // Phase indicator on LCD
  //#define LCD_PHASE_IND false // Software not functional. Do not change!!
  unsigned int MeterMaxVal;
  float WeighedHigh = 32.0;

// Led I/O definition
  const int SINGLEL = 0;
  const int SINGLEH = 1;
  const int TRISTATE = 2;
  const int SINGLE_OFF = 0;
  const int SINGLE_ON = 1;
  const int BICOL_OFF = 2;
  const int BICOLA_ON = 3;
  const int BICOLB_ON = 4;


  // Led mode kan LOW zijn voor led aan de plus, HIGH voor led aan massa en TRISTATE voor bi-cokor led
  const int Led1 = 5;
  byte Led1Mode = SINGLEL;
  byte Led1Stat = SINGLE_ON;
  const int Led2 = 6;
  byte Led2Mode = SINGLEL;
  byte Led2Stat = SINGLE_ON;
  const int Led3 = 7;
  byte Led3Mode = SINGLEL;
  byte Led3Stat = SINGLE_ON;
  byte LockStatus = 0;
  byte OldLockStatus = 0;
  
// Si5351A print GPSDO4
  byte Si5351_addr = 0x60;  // bus address of the I2C Si5351 chip
  unsigned long long Si5351_reference = 10000000; // 10MHz reference input for the chip
  unsigned long long Si0Freq, Si1Freq, Si2Freq;
  unsigned int Si0MHertz, Si0kHertz, Si0Hertz, Si0mHertz;
  unsigned int Si1MHertz, Si1kHertz, Si1Hertz, Si1mHertz;
  unsigned int Si2MHertz, Si2kHertz, Si2Hertz, Si2mHertz;
  char NEW_SI_0_HERTZ[] = "              ", NEW_SI_1_HERTZ[] = "              ", NEW_SI_2_HERTZ[] = "              ";
  char OLD_SI_0_HERTZ[] = "              ", OLD_SI_1_HERTZ[] = "              ", OLD_SI_2_HERTZ[] = "              ";
  byte Si0Level, Si1Level, Si2Level;

// ADF4351 via GPSDO4
  unsigned long ADF4351_reference = 10000000; // 10MHz reference input for the chip
  unsigned long AdfStep = 1000;
  unsigned int  AdfGHz, AdfMHz, AdfkHz, AdfHz;
  unsigned long long AdfFreq;
  //unsigned long AdfFreq;
/*
char NEW_ADF_HERTZ[] = "0.000.000.000";
*/
char NEW_ADF_HERTZ[] = "             ";
char OLD_ADF_HERTZ[] = "             ";
//byte AdfOut;
byte AdfLevel;
//  #define ADF4351_adjust 0 // 0 = off - 1 = on 1 = frequency adjustable by encoder.
//  #define ADF4351_status 0 // 0 = off - 1 = on
//  #define ADF4351_freq 30750000 // 307.5MHz start frequency for Lo. Step size 100Hz
//  #define ADF4351_level 0 // Set output level 0 .. 3 (-4dBM, -1dBm, +2dBm, +5dBm)
//  #define ADF4351_step 10000; //Channel grid 1Khz

String GpsRx = "";

  //eof
